

class Aoi:
    def __init__(self,type_of_element,x,width,y,height,aoi_char):
        self.type_of_element = type_of_element
        self.x = x
        self.width = width
        self.y = y
        self.height = height
        self.aoi_char = aoi_char
